﻿using System;
using System.Collections.Generic;

using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 커스텀 정보 데이터
    /// </summary>
    public class CustomInformationData : IInformationData
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 데이터 응답시 이벤트 - OnDataResponse

        /// <summary>
        /// 데이터 응답시 이벤트
        /// </summary>
        public event EventHandler<RequestCompletedEventArgs> OnDataResponse;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 위치 정보 리스트
        /// </summary>
        private readonly List<LocationInformation> locationInformationList = new List<LocationInformation>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 위치 정보 열거 가능 목록 - LocationInformationList

        /// <summary>
        /// 위치 정보 열거 가능 목록
        /// </summary>
        public IEnumerable<LocationInformation> LocationInformationList
        {
            get
            {
                return this.locationInformationList;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method

        #region 검색하기 - Search(keyword)

        /// <summary>
        /// 검색하기
        /// </summary>
        /// <param name="keyword">키워드</param>
        public void Search(string keyword)
        {
            Random random = new Random(DateTime.Now.Millisecond);

            this.locationInformationList.Clear();

            int length = keyword.Length;

            for(int i = 0; i < length; i++)
            {
                LocationInformation locationInformation = new LocationInformation();

                locationInformation.Address  = new Address(keyword + " " + i.ToString());
                locationInformation.Location = new GeoPoint(random.Next(180) - 90, random.Next(360) - 180);

                locationInformationList.Add(locationInformation);
            }

            FireOnDataResponseEvent();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 데이터 응답시 이벤트 발생시키기 - FireOnDataResponseEvent()

        /// <summary>
        /// 데이터 응답시 이벤트 발생시키기
        /// </summary>
        protected void FireOnDataResponseEvent()
        {
            OnDataResponse?.Invoke(this, CreateEventArgs());
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 이벤트 인자 생성하기 - CreateEventArgs()

        /// <summary>
        /// 이벤트 인자 생성하기
        /// </summary>
        /// <returns>요청 완료시 이벤트 인자</returns>
        private RequestCompletedEventArgs CreateEventArgs()
        {
            MapItem[] mapItemArray = new MapItem[this.locationInformationList.Count];

            for(int i = 0; i < mapItemArray.Length; i++)
            {
                mapItemArray[i] = new MapCallout()
                {
                    Location = this.locationInformationList[i].Location,
                    Text     = this.locationInformationList[i].Address.FormattedAddress
                };
            }

            return new RequestCompletedEventArgs(mapItemArray, null, false, null);
        }

        #endregion
    }
}